/**
 * Copyright (©) 2023 Pinpoint Designs LTD. All right's reserved.
 *
 * @category    Pinpoint
 * @copyright   Copyright (c) 2023 Pinpoint Designs. (https://www.pinpointdesigns.co.uk)
 * @author      Ricky Goacher <ricky.goacher@pinpointdesigns.co.uk>
 */

define([
    'Magento_PageBuilder/js/events',
    'Magento_PageBuilder/js/content-type/preview-collection',
    'Magento_PageBuilder/js/content-type-factory',
    'Magento_PageBuilder/js/config',
    'Magento_PageBuilder/js/content-type-menu/option'
], function (events, PreviewCollection, createContentType, pageBuilderConfig, option) {
    'use strict';

    /**
     * @param parent
     * @param config
     * @param stageId
     * @constructor
     */

    function Preview(parent, config, stageId) {
        PreviewCollection.call(this, parent, config, stageId);
    }

    Preview.prototype = Object.create(PreviewCollection.prototype);

    /**
     * Root element
     */

    Preview.prototype.element = null;

    /**
     * Bind events to add empty USP Item
     */

    Preview.prototype.bindEvents = function bindEvents() {
        var self = this;

        PreviewCollection.prototype.bindEvents.call(this);

        /**
         * Generate USP items when adding USP bar
         */

        events.on("pinpoint_pagebuilderuspbar:dropAfter", (args) => {
            if (args.id === self.contentType.id && self.contentType.children().length === 0) {
                for(let i = 0; i < 4; i++) {
                    self.addUspItem();
                }
            }
        });

        /**
         * Control what happens after USP bar render
         */

        events.on("pinpoint_pagebuilderuspbar:renderAfter", function (args) {
            if (args.id === self.contentType.id) {
                var prevArray = [[...self.contentType.children()], []];

                self.contentType.collection.children.subscribe(function(_this) {
                    prevArray[0] = prevArray[1];
                    prevArray[1] = [..._this];

                    // Create num of child attribute

                    self.contentType.dataStore.set(
                        "data-num-of-child-elements",
                        _this.length,
                    );

                    // Prevent more than 6 USP items

                    if(_this.length > 6) {

                        var result = prevArray[1].filter(function (o1) {
                            return !prevArray[0].some(function (o2) {
                                return o1.id === o2.id;
                            });
                        });

                        var itemToRemove = _this.findIndex(x => x.id === result[0].id);

                        _this[itemToRemove].destroy();
                    }
                });
            }
        });
    };

    /**
     * Add USP item
     */

    Preview.prototype.addUspItem = function () {
        var self = this;
        if(this.contentType.collection.children._latestValue.length < 6) {
            createContentType(
                pageBuilderConfig.getContentTypeConfig("pinpoint_pagebuilderuspitem"),
                this.contentType,
                this.contentType.stageId
            ).then(function (container) {
                self.contentType.addChild(container);
                self.addUspIcon(container);
                self.addUspText(container);
            });
        }
    };

    /**
     * Add USP icon
     */

    Preview.prototype.addUspIcon = function (contentType) {
        createContentType(
            pageBuilderConfig.getContentTypeConfig("pinpoint_pagebuilderuspitemicon"),
            contentType,
            contentType.id
        ).then(function (container) {
            contentType.addChild(container);
        });
    };

    /**
     * Add USP text
     */

    Preview.prototype.addUspText = function (contentType) {
        createContentType(
            pageBuilderConfig.getContentTypeConfig("pinpoint_pagebuilderuspitemtext"),
            contentType,
            contentType.id
        ).then(function (container_2) {
            contentType.addChild(container_2);
        });
    };

    /**
     * Return content menu options
     *
     * @returns {object}
     */

    Preview.prototype.retrieveOptions = function () {
        var self = this;
        var options = PreviewCollection.prototype.retrieveOptions.call(this);

        options.add = new option({
            preview: this,
            icon: "<i class='icon-pagebuilder-add'></i>",
            title: "Add",
            action: self.addUspItem,
            classes: ["add-child"],
            sort: 10
        });
        return options;
    };

    /**
     * Set root element
     *
     * @returns {void}
     */

    Preview.prototype.afterRender = function (element) {
        this.element = element;
    };

    /**
     * Check if content type is container
     *
     * @returns {boolean}
     */

    Preview.prototype.isContainer = function () {
        return true;
    };

    return Preview;
});
