/**
 * Copyright (©) 2023 Pinpoint Designs LTD. All right's reserved.
 *
 * @category    Pinpoint
 * @copyright   Copyright (c) 2023 Pinpoint Designs. (https://www.pinpointdesigns.co.uk)
 * @author      Ricky Goacher <ricky.goacher@pinpointdesigns.co.uk>
 */

define([
    'Magento_PageBuilder/js/events',
    'Magento_PageBuilder/js/content-type/preview-collection',
    'Magento_PageBuilder/js/content-type-factory',
    'Magento_PageBuilder/js/config',
], function (events, PreviewCollection, createContentType, pageBuilderConfig) {
    'use strict';

    /**
     * @param parent
     * @param config
     * @param stageId
     * @constructor
     */

    function Preview(parent, config, stageId) {
        PreviewCollection.call(this, parent, config, stageId);
    }

    Preview.prototype = Object.create(PreviewCollection.prototype);

    /**
     * Root element
     */

    Preview.prototype.element = null;

    /**
     * Bind events to add USP icon & USP text
     */

    Preview.prototype.bindEvents = function bindEvents() {
        var self = this;

        PreviewCollection.prototype.bindEvents.call(this);

        events.on("pinpoint_pagebuilderuspitem:dropAfter", (args) => {
            if (args.id === self.contentType.id && self.contentType.children().length === 0) {
                self.addUspIcon();
                self.addUspText();
            }
        });
    };

    /**
     * Add USP text
     */

    Preview.prototype.addUspText = function () {
        var self = this;
        createContentType(
            pageBuilderConfig.getContentTypeConfig("pinpoint_pagebuilderuspitemtext"),
            this.contentType,
            this.contentType.stageId
        ).then(function (container) {
            self.contentType.addChild(container);
        });
    };

    /**
     * Add USP icon
     */

    Preview.prototype.addUspIcon = function () {
        var self = this;
        createContentType(
            pageBuilderConfig.getContentTypeConfig("pinpoint_pagebuilderuspitemicon"),
            this.contentType,
            this.contentType.stageId
        ).then(function (container) {
            self.contentType.addChild(container);
        });
    };

    /**
     * Set root element
     *
     * @returns {void}
     */

    Preview.prototype.afterRender = function (element) {
        this.element = element;
    };

    /**
     * Check if content type is container
     *
     * @returns {boolean}
     */

    Preview.prototype.isContainer = function () {
        return true;
    };

    return Preview;
});
